@@ -692,15 +692,14 @@ static void _archive_remove (FRArchive *archive, GList *file_list);
 
 
 static GList *
-escape_file_list (GList *file_list) {
+escape_file_list (GList *file_list) 
+{
 	GList *e_file_list = NULL;
 	GList *scan;
 
 	for (scan = file_list; scan; scan = scan->next) {
-		gchar *filename = scan->data;
-
-		e_file_list = g_list_prepend (e_file_list, 
-					      shell_escape (filename));
+		char *filename = scan->data;
+		e_file_list = g_list_prepend (e_file_list, shell_escape (filename));
 	}
 
 	return e_file_list;
