@@ -91,6 +91,36 @@ mktime_from_string (char *date_s,
 }
 
 
+static char*
+zip_escape (const char *str)
+{
+	return escape_str (str, "*?[]");
+}
+
+
+static char*
+prepend_path_separator (const char *str)
+{
+	if (*str == '-' || g_str_has_prefix (str, "\\-")) 
+		return g_strconcat (".", G_DIR_SEPARATOR_S, str, NULL);
+	else 
+		return g_strdup (str);
+}
+
+
+static char*
+prepend_path_separator_zip_escape (const char *str)
+{
+	char *tmp1, *tmp2;
+
+	tmp2 = prepend_path_separator (str);
+	tmp1 = zip_escape (tmp2);
+	g_free (tmp2);
+	
+	return tmp1;
+}
+
+
 static void
 list__process_line (char     *line, 
 		    gpointer  data)
@@ -150,9 +180,41 @@ list__process_line (char     *line,
 }
 
 
+static void
+add_filename_arg (FRCommand *comm) 
+{
+	char *temp = prepend_path_separator (comm->e_filename);
+	fr_process_add_arg (comm->process, temp);
+	g_free (temp);
+}
+
+
+static void
+add_password_arg (FRCommand     *comm,
+		  const char    *password)
+{
+	fr_process_add_arg (comm->process, "-P");
+
+	if (password != NULL) {
+		char *arg;
+		char *e_password;
+
+		e_password = escape_str (password, "\"");
+		arg = g_strconcat ("\"", e_password, "\"", NULL);
+		g_free (e_password);
+
+		fr_process_add_arg (comm->process, arg);
+		g_free (arg);
+	} else
+		fr_process_add_arg (comm->process, "\"\"");
+}
+
+
 static void
 fr_command_zip_list (FRCommand *comm)
 {
+
+
 	FR_COMMAND_ZIP (comm)->is_empty = FALSE;
 
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
@@ -164,7 +226,7 @@ fr_command_zip_list (FRCommand *comm)
 	fr_process_add_arg (comm->process, "-qq");
 	fr_process_add_arg (comm->process, "-v");
 	fr_process_add_arg (comm->process, "-l");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	add_filename_arg (comm);
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
 }
@@ -214,10 +276,7 @@ fr_command_zip_add (FRCommand     *comm,
 	if (update)
 		fr_process_add_arg (comm->process, "-u");
 
-	if (password != NULL) {
-		fr_process_add_arg (comm->process, "-P");
-		fr_process_add_arg (comm->process, password);
-	}
+	add_password_arg (comm, password);
 
 	switch (compression) {
 	case FR_COMPRESSION_VERY_FAST:
@@ -230,10 +289,13 @@ fr_command_zip_add (FRCommand     *comm,
 		fr_process_add_arg (comm->process, "-9"); break;
 	}
 
-	fr_process_add_arg (comm->process, comm->e_filename);
+	add_filename_arg (comm);
 
-	for (scan = file_list; scan; scan = scan->next) 
-		fr_process_add_arg (comm->process, (gchar*) scan->data);
+	for (scan = file_list; scan; scan = scan->next) {
+		char *temp = prepend_path_separator_zip_escape ((char*) scan->data);
+		fr_process_add_arg (comm->process, temp);
+		g_free (temp);
+	}
 
 	fr_process_end_command (comm->process);
 }
@@ -251,10 +313,14 @@ fr_command_zip_delete (FRCommand *comm,
 
 	fr_process_begin_command (comm->process, "zip");
 	fr_process_add_arg (comm->process, "-d");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	add_filename_arg (comm);
+
+	for (scan = file_list; scan; scan = scan->next) {
+		char *temp = prepend_path_separator_zip_escape ((char*) scan->data);
+		fr_process_add_arg (comm->process, temp);
+		g_free (temp);
+	}
 
-	for (scan = file_list; scan; scan = scan->next)
-		fr_process_add_arg (comm->process, scan->data);
 	fr_process_end_command (comm->process);
 }
 
@@ -277,7 +343,7 @@ fr_command_zip_extract (FRCommand  *comm,
 	fr_process_begin_command (comm->process, "unzip");
 	
 	if (dest_dir != NULL) {
-		gchar *e_dest_dir = shell_escape (dest_dir);
+		char *e_dest_dir = shell_escape (dest_dir);
 		fr_process_add_arg (comm->process, "-d");
 		fr_process_add_arg (comm->process, e_dest_dir);
 		g_free (e_dest_dir);
@@ -294,16 +360,16 @@ fr_command_zip_extract (FRCommand  *comm,
 	if (junk_paths)
 		fr_process_add_arg (comm->process, "-j");
 
-	if (password != NULL) {
-		fr_process_add_arg (comm->process, "-P");
-		fr_process_add_arg (comm->process, password);
+	add_password_arg (comm, password);
+
+	add_filename_arg (comm);
+
+	for (scan = file_list; scan; scan = scan->next) {
+		char *temp = prepend_path_separator_zip_escape ((char*) scan->data);
+		fr_process_add_arg (comm->process, temp);
+		g_free (temp);
 	}
 
-	fr_process_add_arg (comm->process, comm->e_filename);
-
-	for (scan = file_list; scan; scan = scan->next)
-		fr_process_add_arg (comm->process, scan->data);
-
 	fr_process_end_command (comm->process);
 }
 
@@ -314,13 +380,8 @@ fr_command_zip_test (FRCommand   *comm,
 {
 	fr_process_begin_command (comm->process, "unzip");
 	fr_process_add_arg (comm->process, "-t");
-
-	if (password != NULL) {
-		fr_process_add_arg (comm->process, "-P");
-		fr_process_add_arg (comm->process, password);
-	}
-
-	fr_process_add_arg (comm->process, comm->e_filename);
+	add_password_arg (comm, password);
+	add_filename_arg (comm);
 	fr_process_end_command (comm->process);
 }
 
@@ -329,8 +390,7 @@ static void
 fr_command_zip_handle_error (FRCommand   *comm, 
 			     FRProcError *error)
 {
-	if ((error->type == FR_PROC_ERROR_GENERIC) 
-	    && (error->status <= 1))
+	if ((error->type == FR_PROC_ERROR_GENERIC) && (error->status <= 1))
 		error->type = FR_PROC_ERROR_NONE;
 }
 
